@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 - 2007 by                                          *
+ *   Copyright (C) 2005 - 2008 by                                          *
  *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
  *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
  *      Jono Cole, Last.fm Ltd <jono@last.fm>                              *
@@ -23,13 +23,14 @@
 #include "lastfmapplication.h"
 
 #include "configwizard.h"
+#include "confirmdialog.h"
 #include "container.h"
 #include "LastMessageBox.h"
 #include "libFingerprint/FingerprintCollector.h"
 #include "libFingerprint/FingerprintQueryer.h"
 #include "logger.h"
 #include "loginwidget.h"
-#include "MediaDeviceConfirmDialog.h"
+#include "MediaDeviceScrobbler.h"
 #include "playercommands.h"
 #include "playerlistener.h"
 #include "User.h"
@@ -39,8 +40,12 @@
 #include "Scrobbler-1.2.h"
 #include "LastFmSettings.h"
 #include "version.h"
-#include "Bootstrapper/PluginBootstrapper.h"
 #include "CachedHttpJanitor.h"
+#include "interfaces/ExtensionInterface.h"
+
+#ifndef Q_WS_X11
+    #include "Bootstrapper/PluginBootstrapper.h"
+#endif
 
 #include "MooseCommon.h"
 #include "UnicornCommon.h"
@@ -57,6 +62,7 @@
 
 #ifdef Q_WS_MAC
     #include "itunesscript.h"
+    #include "ITunesPluginInstaller.h"
     #include <Carbon/Carbon.h>
 #endif
 
@@ -68,7 +74,8 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
           m_user( 0 ),
           m_activeNorman( 0 ),
           m_state( State::Stopped ),
-          m_proxyTestDone( false )
+          m_proxyTestDone( false ),
+          m_extensionsLoaded( false )
 {
     #ifdef Q_WS_MAC
         m_pidFile.setFileName( MooseUtils::savePath( "lastfm.pid" ) );
@@ -84,7 +91,10 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     // TODO why do we do this, and do we need to?
     //   Good question, this was added as a hacky workaround to try and fix
     //   some QHttp wonkiness we had on some machines back in the day. Best
-    //   not removing it.
+    //   not removing it. May fix obscure bugs like, ws.as.com is not accessible
+    //   for 3 or 4 minutes
+    // NOTE it's prolly a good thing for feedback reasons anyway, the UI appears
+    //   quicker and thus feels like it starts faster
     QTimer::singleShot( 0, this, SLOT( init() ) );
 
     // We're doing this here because the version.h file is updated just before
@@ -98,13 +108,9 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     CachedHttp::setCustomUserAgent( QString( "Last.fm Client " ) + LASTFM_CLIENT_VERSION );
     CachedHttp::setCustomCachePath( MooseUtils::cachePath() );
 
-    // Start helper daemon _before_ we bind any sockets, otherwise they might
-    // end up being blocked by the child-process.
-    MooseUtils::installHelperApp();
-
     // do this asap to prevent multiple instances instantiating
     m_control = new QTcpServer( this );
-    connect( m_control, SIGNAL(newConnection()), SLOT(onControlConnect()) );
+    connect( m_control, SIGNAL( newConnection() ), SLOT( onControlConnect() ) );
 
     // Try to use the default port, otherwise revert to using an available port
     // and write the port we're using to the Settings. sendToInstance() will use
@@ -114,7 +120,7 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
 
     The::settings().setControlPort( m_control->serverPort() );
 
-    connect( this, SIGNAL(event( int, QVariant )), SLOT(onAppEvent( int, QVariant )) );
+    connect( this, SIGNAL( event( int, QVariant ) ), SLOT( onAppEvent( int, QVariant ) ) );
 
     initLogger();
     initTranslator();
@@ -125,6 +131,10 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     setWindowIcon( QIcon( MooseUtils::dataPath( "icons/as.ico" ) ) );
   #endif
 
+  #ifdef WIN32
+    MooseUtils::disableHelperApp();    
+  #endif
+
     // This is needed so that relative paths will work on Windows regardless
     // of where the app is launched from.
     QDir::setCurrent( applicationDirPath() );
@@ -132,20 +142,9 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     // this must be set before dialogs spawn in init() or whatever
     m_user = new User( this );
 
-    //REFACTOR this class merged here!
+    // TODO this class merged here!
     The::webService(); //init webservice stuff
-    connect( The::webService(), SIGNAL(result( Request* )), SLOT(onRequestReturned( Request* )) );
-
-    // HACK: this is only here to fix the bug where the initial wizard runs and we're behind a proxy.
-    // We need to have done these tests before running the VerifyUserRequest, otherwise the Http object
-    // will not pick an autodetected proxy if there is one. Refactor for 1.5.
-    if( !The::settings().isUseProxy() )
-    {
-        ProxyTestRequest* proxyOnTest = new ProxyTestRequest( true );
-        ProxyTestRequest* proxyOffTest = new ProxyTestRequest( false );
-        proxyOnTest->start();
-        proxyOffTest->start();
-    }
+    connect( The::webService(), SIGNAL( result( Request* ) ), SLOT( onRequestReturned( Request* ) ) );
 
     // Scrobbler must be initialised before the listener, otherwise crashes might ensue
     m_scrobbler = new ScrobblerManager( this );
@@ -168,16 +167,16 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     new ITunesScript( this, m_listener );
   #endif
 
-    m_fpCollector = new FingerprintCollector( 1, this );
+    m_fpCollector = new FingerprintCollector( 1 /*number of threads*/, this );
     m_fpQueryer = new FingerprintQueryer( this );
     connect( m_fpQueryer, SIGNAL( trackFingerprinted( TrackInfo, bool ) ),
                           SLOT( onFingerprintQueryDone( TrackInfo, bool ) ) );
 
     m_radio = new Radio( this );
-    connect( m_radio, SIGNAL(stateChanged( RadioState )), SLOT(onRadioStateChanged( RadioState )) );
+    connect( m_radio, SIGNAL( stateChanged( RadioState ) ), SLOT( onRadioStateChanged( RadioState ) ) );
 
     m_container = new Container;
-    connect( m_container, SIGNAL(becameVisible()), SLOT(fetchMetaData()) );
+    connect( m_container, SIGNAL( becameVisible() ), SLOT( fetchMetaData() ) );
 
     // Look for expired cached files and remove them
     new CachedHttpJanitor( MooseUtils::cachePath(), this );
@@ -185,7 +184,11 @@ LastFmApplication::LastFmApplication( int& argc, char** argv )
     m_playbackEndedTimer = new QTimer( this );
     m_playbackEndedTimer->setSingleShot( true );
     m_playbackEndedTimer->setInterval( 200 );
-    connect( m_playbackEndedTimer, SIGNAL(timeout()), SLOT(onPlaybackEndedTimerTimeout()) );
+    connect( m_playbackEndedTimer, SIGNAL( timeout() ), SLOT( onPlaybackEndedTimerTimeout() ) );
+
+    // This is needed for the app to shut down properly when asked by Windows
+    // to shut down.
+    connect( this, SIGNAL( endSession() ), SLOT( quit() ) );
 }
 
 
@@ -203,18 +206,38 @@ LastFmApplication::init()
     }
   #endif
 
+
     foreach ( QString const arg, qApp->arguments().mid( 1 ) ) //skip arg[0] the appname+path
         parseCommand( arg );
 
     // Need to save the state from before we run the wizard as the wizard will change it
     bool firstRunBeforeWizard = The::settings().isFirstRun();
 
+    #ifdef Q_OS_MAC
+      // the installation of the plugin is done in the ctor
+      ITunesPluginInstaller pluginInstaller;
+      pluginInstaller.install();
+    #endif
+
+
     if ( The::settings().isFirstRun() )
     {
-        LOG( 3, "First run, launching config wizard\n" );
+        // HACK: this is only here to fix the bug where the initial wizard runs and we're behind a proxy.
+        // We need to have done these tests before running the VerifyUserRequest, otherwise the Http object
+        // will not pick an autodetected proxy if there is one. Refactor for 1.5.
+        if( The::settings().isFirstRun() && !The::settings().isUseProxy() )
+        {
+            ProxyTestRequest* proxyOnTest = new ProxyTestRequest( true );
+            ProxyTestRequest* proxyOffTest = new ProxyTestRequest( false );
+            proxyOnTest->start();
+            proxyOffTest->start();
+        }
+
+        LOGL( 3, "First run, launching config wizard" );
         QFile( MooseUtils::savePath( "mediadevice.db" ) ).remove();
 
         ConfigWizard wiz( NULL, ConfigWizard::Login );
+
         if ( wiz.exec() == QDialog::Rejected )
         {
             // If user cancels config wizard, we need to exit
@@ -222,6 +245,51 @@ LastFmApplication::init()
             return;
         }
     }
+  #ifndef LINUX
+    else
+    {
+      #ifdef Q_OS_MAC
+        // We just installed the plugin for the first time
+        bool const needsBootstrap = pluginInstaller.needsTwiddlyBootstrap();
+      #else
+        // The update wizard invites the user to upgrade the iTunes plugin, and
+        // it did this last time the client was invoked. iTunes has been 
+        // stoppped, so the new plugin will be running when we Twiddly starts
+        // iTunes during the bootstrap
+        bool needsBootstrap = The::settings().weWereJustUpgraded();
+      #endif
+        
+        //NOTE some of this code is duplicated in Container::updateCheckDone()
+        if ( needsBootstrap )
+        {
+            // the NULL is strange, and I have no idea why we do it here or above
+            // hopefully whoever did it in the first place will comment it :P
+            ConfigWizard( NULL, ConfigWizard::MediaDevice ).exec();
+        }
+    }
+  #endif
+
+  #ifdef QT_NO_DEBUG
+  #ifndef LINUX
+    {
+        qDebug() << "Preferred application for lastfm url-scheme:" << UnicornUtils::preferredAppForUrlScheme( QUrl( "lastfm://" ) );
+        qDebug() << "Our app:" << MooseUtils::bundleDirPath();
+
+        bool b = ( MooseUtils::bundleDirPath() == UnicornUtils::preferredAppForUrlScheme( QUrl( "lastfm://" ) ) );
+        if ( !b )
+        {
+            ConfirmDialog d( m_container );
+            d.setText( tr( "The Last.fm Client is currently not the default player for Last.fm streams.\nDo you want to change that? *CHANEGCOPY*" ) );
+            d.setOperationString( "urlhandler" );
+
+            if ( d.exec() )
+            {
+                qDebug() << "Setting client as the default lastfm url-scheme handler worked:" << UnicornUtils::setPreferredAppForUrlScheme( QUrl( "lastfm://" ), MooseUtils::bundleDirPath() );
+            }
+        }
+    }
+  #endif
+  #endif
 
     // Do we have a current user?
     QString currentUser = The::settings().currentUsername();
@@ -248,12 +316,9 @@ LastFmApplication::init()
         }
     }
     else
+    {
         setUser( currentUser );
-
-    // This is needed for the app to shut down properly when asked by Windows
-    // to shut down.
-    //TODO move to ctor?
-    connect( this, SIGNAL( endSession() ), SLOT( quit() ) );
+    }
 
     if ( !arguments().contains( "-tray" ) && !arguments().contains( "--tray" ) )
         m_container->show();
@@ -327,7 +392,8 @@ LastFmApplication::initLogger()
     logger.Init( MooseUtils::logPath( filename ), false );
     logger.SetLevel( Logger::Debug );
 
-    LOGL( 1, "App version: " << The::settings().version() );
+    LOGL( 3, "App version: " << The::settings().version() );
+    LOGL( 3, "OS version: " << UnicornUtils::getOSVersion() );
 }
 
 
@@ -383,10 +449,19 @@ LastFmApplication::registerMetaTypes()
 void
 LastFmApplication::setUser( const QString& username )
 {
+    Q_ASSERT( !username.isEmpty() );
+    
     The::settings().setCurrentUsername( username );
     The::webService()->setUsername( username );
     The::webService()->setPassword( The::currentUser().password() );
 
+    if ( !m_extensionsLoaded )
+    {
+        // Can't load extensions until we have a current user
+        loadExtensions();
+        m_extensionsLoaded = true;
+    }
+
     if ( m_user )
     {
         // we no longer care about any signals this user may emit
@@ -397,8 +472,6 @@ LastFmApplication::setUser( const QString& username )
     m_handshaked = false;
     m_user = new User( username, this );
 
-// CAREFUL: this ifndef spawns and ends in the next method (onProxyTestComplete)
-// FIXME: oh please fix me
 #ifndef Q_WS_X11
     m_proxyTestDone = false;
     if( !The::settings().isUseProxy() )
@@ -407,8 +480,10 @@ LastFmApplication::setUser( const QString& username )
         ProxyTestRequest* proxyOnTest = new ProxyTestRequest( true );
         ProxyTestRequest* proxyOffTest = new ProxyTestRequest( false );
 
-        connect( The::webService() , SIGNAL( proxyTestResult( bool ) ),
-                 this,               SLOT( onProxyTestComplete( bool ) ) );
+        disconnect( The::webService(), SIGNAL( proxyTestResult( bool, WebRequestResultCode ) ),
+                    this,              SLOT( onProxyTestComplete( bool, WebRequestResultCode ) ) );
+        connect( The::webService(), SIGNAL( proxyTestResult( bool, WebRequestResultCode ) ),
+                this,              SLOT( onProxyTestComplete( bool, WebRequestResultCode ) ) );
 
         proxyOnTest->start();
         proxyOffTest->start();
@@ -417,30 +492,44 @@ LastFmApplication::setUser( const QString& username )
     else
     {
         emit event( Event::UserChanged, username );
-        onProxyTestComplete( false );
+        onProxyTestComplete( false, Request_Success );
     }
-
-    emit event( Event::UserChanged, username );
+#else
+    m_proxyTestDone = false;
+    onProxyTestComplete( false, Request_Success  );
+#endif
 }
 
 
 void
-LastFmApplication::onProxyTestComplete( bool proxySet )
+LastFmApplication::onProxyTestComplete( bool proxySet, WebRequestResultCode result, bool authProxyTimerComplete )
 {
     // The only reason for this slot is to delay the handshake until we know whether
     // to use the proxy or not.
+    qDebug() << "*********** ProxyTest Complete Result code = " << result;
+
     
+    if( result == Request_ProxyAuthenticationRequired && !authProxyTimerComplete )
+    {
+        // Proxy authentication required.
+        // Wait 3 seconds to see if a direct connection is available before 
+        // prompting the user for login details.
+        QTimer::singleShot( 3000, this, SLOT( onAuthenticatedProxyTimerTimeout() ) );
+        return;
+    }
+
     if ( m_proxyTestDone ) return;
     m_proxyTestDone = true;
-
+    
+#ifndef Q_WS_X11
     LOGL( 3, ( proxySet ? "" : "not "  ) << "using autodetected proxy settings" );
+#endif
 
     // HACK: since we're in a different function, we need to set the username again
     // as it was a parameter to setUser.
     QString username = m_user->settings().username();
-#else
+
     emit event( Event::UserChanged, username );
-#endif
 
     QString password = m_user->settings().password();
     QString version = The::settings().version();
@@ -452,6 +541,7 @@ LastFmApplication::onProxyTestComplete( bool proxySet )
     m_fpQueryer->setUsername( username );
     m_fpQueryer->setPasswordMd5( password );
     m_fpQueryer->setPasswordMd5Lower( password ); // FIXME: surely they can't be the same!
+    m_fpQueryer->setVersion( The::settings().version() );
 
     // init radio YTIO
     m_radio->init( username, password, version );
@@ -477,6 +567,50 @@ LastFmApplication::onProxyTestComplete( bool proxySet )
 }
 
 
+////////////////////////////////////////////////////////////////////////////
+// After a timeout period this will determine if a direct connection has  // 
+// been established by another proxyTestRequest. If not then              //
+// authentication required dialog is shown before resuming user handshake.//
+////////////////////////////////////////////////////////////////////////////
+void
+LastFmApplication::onAuthenticatedProxyTimerTimeout()
+{
+    if( !m_proxyTestDone )
+    {
+        LastMessageBox::information( tr( "Proxy Authentication Required" ),
+            tr( "The proxy autodetection has detected a proxy server but does not have enough information to authenticate.\n\n"
+            "Please set the proxy settings to manual and enter the username and password required." ) );
+
+        The::container().showSettingsDialog( 3 );
+        onProxyTestComplete( true, Request_ProxyAuthenticationRequired, true );
+    }
+}
+
+
+void
+LastFmApplication::loadExtensions()
+{
+    foreach( QString path, Moose::extensionPaths() )
+    {
+        LOGL( 3, "Loading extension: " << path );
+        QObject* plugin = QPluginLoader( path ).instance();
+        if ( !plugin )
+	    {
+            LOGL( 1, "Failed to load " << path );
+            continue;
+        }
+
+        LOGL( 3, "Extension loaded" );
+        ExtensionInterface* iExtension = qobject_cast<ExtensionInterface *>( plugin );
+        if ( iExtension )
+        {
+            QSettings* us = new CurrentUserSettings( this );
+            iExtension->setSettings( us );
+        }
+    }
+}
+
+
 #ifdef WIN32
 bool
 LastFmApplication::winEventFilter( MSG * msg, long * result )
@@ -587,11 +721,6 @@ LastFmApplication::onPlaybackEndedTimerTimeout()
 void
 LastFmApplication::onScrobblePointReached( const TrackInfo& track )
 {
-    if ( track.playerId() == "itw" || track.playerId() == "osx" )
-    {
-        ItunesScrobbleHistory().append( track );
-    }
-
     emit event( Event::ScrobblePointReached, QVariant::fromValue( track ) );
 }
 
@@ -602,73 +731,14 @@ LastFmApplication::onScrobblerStatusUpdate( int code, const QVariant& data )
     if ( code == Scrobbler::Handshaken )
     {
         QString const username = data.toString();
-        ScrobbleCache cache = ScrobbleCache::mediaDeviceCache( username );
-        // we show a nothing to scrobble message if empty usually
-        // but this is an automated check, and not user-triggered
-        if ( !cache.tracks().isEmpty() )
-            scrobbleMediaDeviceCache( username );
+        IPodScrobbler( username, m_container ).exec();
     }
 }
 
 
-void
-LastFmApplication::scrobbleMediaDeviceCache( const QString& username )
-{
-    ScrobbleCache icache = ScrobbleCache::mediaDeviceCache( username );
-    ScrobbleCache cache( username );
-
-    if ( icache.tracks().isEmpty() )
-    {
-        //FIXME messages that interupt suck
-        LastMessageBox::information( tr( "Nothing To Scrobble" ),
-                                     tr( "Your iPod has nothing new to scrobble." ) );
-        return;
-    }
-
-    MediaDeviceConfirmDialog dialog( username, m_container );
-    if ( dialog.exec() == QDialog::Accepted )
-    {
-        // copy tracks playCount() times so we submit the correct number of plays
-        QList<TrackInfo> tracks;
-        foreach ( TrackInfo const t, dialog.tracks() )
-        {
-            for ( int y = 0; y < t.playCount(); ++y )
-                tracks += t;
-
-            // will add each track (but not each single play!) to the recently listened tracks
-            emit event( Event::MediaDeviceTrackScrobbled, QVariant::fromValue( t ) );
-        }
-
-        cache.append( tracks );
-    }
-
-    //HACK since our automatic sync iPod handling blows, we monitor all
-    // iTunes plays and check we aren't submitting regular iTunes plays
-    QFile f( icache.path() );
-    f.open( QFile::Text | QFile::ReadOnly );
-    QDomDocument xml;
-    xml.setContent( &f );
-    QString const databaseLastModified = xml.documentElement().attribute( "lastItunesUpdate" );
-    f.close();
-
-    // will be empty if not an iPod mediadevice cache
-    if ( !databaseLastModified.isEmpty() )
-    {
-        ItunesScrobbleHistory().prune( databaseLastModified.toUInt() );
-    }
-
-    //FIXME possible race condition since LastFmHelper also writes to this file
-    QFile::remove( icache.path() );
-
-    m_scrobbler->scrobble( cache );
-}
-
-
 void
 LastFmApplication::onRadioStateChanged( RadioState newState )
 {
-  #ifndef HIDE_RADIO
-
     switch ( newState )
     {
         case State_Handshaken:
@@ -737,8 +807,6 @@ LastFmApplication::onRadioStateChanged( RadioState newState )
         default:
             Q_ASSERT( !"Undefined state case reached in onRadioStateChanged!" );
     }
-
-  #endif // HIDE_RADIO
 }
 
 
@@ -778,10 +846,8 @@ LastFmApplication::onAppEvent( int event, const QVariant& /* data */ )
                         LOGL( 2, "Failed to extract MBID for: " << m_currentTrack.path() );
                 }
 
-                if (m_container->isVisible())
-                {
+                if ( m_container->isVisible() )
                     fetchMetaData();
-                }
 
                 if ( QFile::exists( m_currentTrack.path() ) &&
                      The::settings().currentUser().fingerprintingEnabled() )
@@ -861,9 +927,8 @@ LastFmApplication::onRequestReturned( Request* request )
 
             Handshake* handshake = static_cast<Handshake*>(request);
             The::user().m_isSubscriber = handshake->isSubscriber();
-
-            break;
         }
+        break;
 
         case TypeArtistMetaData:
         {
@@ -886,9 +951,8 @@ LastFmApplication::onRequestReturned( Request* request )
 
             // if track has changed before we got this metadata, don't emit the event ;)
             emit event( Event::ArtistMetaDataAvailable, QVariant::fromValue( m_currentTrack ) );
-
-            break;
         }
+        break;
 
         case TypeTrackMetaData:
         {
@@ -918,17 +982,12 @@ LastFmApplication::onRequestReturned( Request* request )
 
             // if track has changed before we got this metadata, don't emit the event ;)
             emit event( Event::TrackMetaDataAvailable, QVariant::fromValue( m_currentTrack ) );
-
-            break;
         }
+        break;
 
         default:
             break;
     }
-
-//TODO
-//    request->abort();
-//    request->deleteLater();
 }
 
 
@@ -952,7 +1011,7 @@ LastFmApplication::onControlConnect()
     // some other program like the firefox extension.
 
     QTcpSocket* socket = m_control->nextPendingConnection();
-    connect( socket, SIGNAL(readyRead()), SLOT(onControlRequest()) );
+    connect( socket, SIGNAL( readyRead() ), SLOT( onControlRequest() ) );
 }
 
 
@@ -991,88 +1050,120 @@ LastFmApplication::parseCommand( const QString& request )
 
     if ( request.contains( "container://show" ) )
     {
-        LOGL( 3, "Calling restoreWindow" );
         m_container->restoreWindow();
     }
 
-    if ( request.contains( "container://checkScrobblerCache" ) )
+  #ifndef Q_WS_X11
+    if ( request.contains( "container://SubmitScrobbleCache/Device" ) )
     {
-        QString const username = request.split( "checkScrobblerCache/" ).value( 1 );
+        QStringList const params = request.split( "container://SubmitScrobbleCache/Device/" ).value( 1 ).split( "/" );
+        int const vid = params.value( 2 ).toInt(); //vendor id
+        int const pid = params.value( 3 ).toInt(); //product id
+        QString const uid = params.value( 0 ) + '/' + params.value( 1 );
+        QString username = The::settings().usernameForDeviceId( uid );
 
-        if (m_scrobbler->canScrobble( username ))
+        if ( username.isEmpty() )
         {
-            scrobbleMediaDeviceCache( username );
+            username = The::currentUsername();
+            The::settings().addMediaDevice( uid, username );
         }
-        else {
-            Scrobbler::Init init;
-            init.username = username;
-            init.password = The::settings().user( username ).password();
-            init.client_version = The::settings().version();
-
-            m_scrobbler->handshake( init );
-
-            // we will submit the mediadevice cache once the scrobbler comes back to us
-        }
-    }
-
-    if ( request.contains( "container://addMediaDevice" ) )
-    {
-        LOGL( 3, "Calling container for media device addition" );
-        addMediaDevice( request.split( "addMediaDevice/" ).at( 1 ) );
-    }
-}
-
-
-/** send to the already running instance of this Application */
-bool
-LastFmApplication::sendToInstance( const QString& data ) //static
-{
-    LOGL( 3, "sendToInstance (new instance): " << data );
-
-    QTcpSocket socket;
-    socket.connectToHost( QHostAddress::LocalHost, The::settings().controlPort() );
-
-    if ( socket.waitForConnected( 500 ) )
-    {
-        if ( data.length() > 0 )
+        
+        if ( The::currentUsername() != username )
         {
-            QByteArray utf8Data = data.toUtf8();
-            socket.write( utf8Data, utf8Data.length() );
-            socket.flush();
+            LastMessageBox::warning( tr("iPod Scrobbling"),
+                                     tr("<p>This iPod is associated with a different Last.fm account."
+                                        "<p>Please log in as <b>%1</b> to submit the scrobbles."
+                                        "<p>You can change the user association in the Options dialog.").arg( username ),
+                                     QMessageBox::Ok,
+                                     QMessageBox::Ok );
+        }
+        else
+        {
+            IPodScrobbler( username, m_container ).exec();
         }
-
-        socket.close();
-        return true;
     }
-    else
+
+    if ( request.startsWith( "container://Notification" ) && !ConfigWizard::isActive() )
     {
-        LOGL( 1, "sendToInstance failed" << data );
+        QString title, message;
+        QStringList keys = request.mid( 12 ).split( '/' );
+
+        if (keys.value( 1 ) == "Twiddly")
+        {
+            title = tr("iPod Scrobbling");
+            QString key = keys.value( 2 );
+                        
+            // ask Toby if you wonder why we don't show any of these messages
+            // by default
+            if (The::currentUser().giveMoreIPodScrobblingFeedback())
+            {
+                if ( key == "Started" )
+                    message = tr( "Your iPod scrobbles are being determined. Please don't exit iTunes." );
+                
+                if ( key == "Error" )
+                {
+                    title = tr( "iPod Scrobbling Error" );
+                    message = keys.value( 3 ).replace( '_', ' ' );
+                }
+            
+                if ( key == "Finished" )
+                {
+                    int const n = keys.value( 3 ).toInt();
+                    if (n)
+                        message = tr( "Last.fm found %n scrobbles on your iPod.", "", n );
+                    else
+                        message = tr( "No scrobbles were found on your iPod." );
+                }
+            }
+            
+            if (key == "Bootstrap")
+            {
+                key = keys.value( 3 );
+                if (key == "Started")
+                    message = tr("Preparing for iPod scrobbling, please don't exit iTunes or sync your iPod.");
+                if (key == "Finished")
+                    message = tr("Last.fm is now ready for iPod scrobbling.");
+            }
+        }
+        else
+        {
+            QString key = keys.value( 2 );
+
+            if ( key == "IPodDetected" )
+            {
+                title = tr( "iPod detected" );
+                message = tr( "Your iPod will be scrobbled to your Last.fm profile from now on." );
+            }
+        }
+        
+        if (!message.isEmpty())
+        {
+            m_container->showNotification( title, message );
+        }
     }
-
-    return false;
-}
-
-
-void
-LastFmApplication::addMediaDevice( const QString& uid )
-{
-    ConfigWizard cw( NULL, ConfigWizard::MediaDevice, uid );
-    if ( !cw.isWizardRunning() )
-        cw.exec();
+  #endif
 }
 
 
 void
 LastFmApplication::onBootstrapReady( QString userName, QString pluginId )
 {
+    #ifndef Q_WS_X11
     // Ignore the bootstrap if we're not currently logged in as the user who initiated it.
     // (The bootstrap file will be detected when the user who initiated it logs back in.)
-    if( userName != The::currentUsername() ) return;
+    if ( userName != The::currentUsername() )
+    {
+        LastMessageBox::warning( "Bootstrap detected",
+            QString( "Bootstrap information was detected for user %1.\n"
+            "Please log in as %1 to submit this." ).arg( userName ),
+            QMessageBox::Ok,
+            QMessageBox::Ok );
+        return;
+    }
 
     PluginBootstrapper* bootstrapper = new PluginBootstrapper( pluginId, this );
-
     bootstrapper->submitBootstrap();
-
+    #endif
 }
 
 
@@ -1083,9 +1174,7 @@ LastFmApplication::onFingerprintQueryDone( TrackInfo track, bool fullFpRequested
     // a metadata request in-between requesting the fp and getting it.
 
     if ( m_currentTrack.path() != track.path() )
-    {
         return;
-    }
 
     m_currentTrack.setFpId( track.fpId() );
 
