@@ -28,23 +28,6 @@
 static QString openUrl;
 
 
-bool
-isAlreadyRunning()
-{
-  #ifdef WIN32
-    QString id( "Lastfm-F396D8C8-9595-4f48-A319-48DCB827AD8F" );
-    ::CreateMutexA( NULL, false, id.toAscii() );
-
-    // The call fails with ERROR_ACCESS_DENIED if the Mutex was 
-    // created in a different users session because of passing
-    // NULL for the SECURITY_ATTRIBUTES on Mutex creation);
-    return ::GetLastError() == ERROR_ALREADY_EXISTS || ::GetLastError() == ERROR_ACCESS_DENIED;
-  #else
-    return LastFmApplication::sendToInstance();
-  #endif
-}
-
-
 /******************************************************************************
     getURLEvent (Apple's way to tell us about cli-args)
 ******************************************************************************/
@@ -68,8 +51,7 @@ getURLEvent( const AppleEvent* appEvent, AppleEvent* /*reply*/, long /*handlerRe
 
     // Send to currently running instance, otherwise variable openUrl
     // will be picked up by the main function
-    if ( isAlreadyRunning() )
-        LastFmApplication::sendToInstance( openUrl );
+    Moose::sendToInstance( openUrl, Moose::DontStartNewInstance );
 
     return noErr;
 }
@@ -80,8 +62,7 @@ getOpenEvent( const AppleEvent* /*appEvent*/, AppleEvent* /*reply*/, long /*hand
 {
     qDebug() << "Receiving apple open event";
 
-    if ( isAlreadyRunning() )
-        LastFmApplication::sendToInstance( "container://show" );
+    Moose::sendToInstance( "container://show", Moose::DontStartNewInstance );
 
     return noErr;
 }
@@ -96,7 +77,7 @@ int main( int argc, char *argv[] )
 {
     #ifndef NBREAKPAD
     BreakPad breakpad( MooseUtils::savePath() );
-    breakpad.setProductName( "Last.fm Client" );
+    breakpad.setProductName( "Moose" );
     #endif
 
     // used by some Qt stuff, eg QSettings
@@ -125,23 +106,21 @@ int main( int argc, char *argv[] )
     }
 
     #ifndef LASTFM_MULTI_PROCESS_HACK
-        if ( isAlreadyRunning() )
+        // Creates the application mutex, so don't ever remove this call!
+        if ( Moose::isAlreadyRunning() )
         {
             if ( !openToTray )
-                LastFmApplication::sendToInstance( "container://show" );
+                Moose::sendToInstance( "container://show" );
 
             if ( !openUrl.isEmpty() )
-                LastFmApplication::sendToInstance( openUrl );
+                Moose::sendToInstance( openUrl );
 
-            //FIXME shouldn't we return 0? Since we exit successfully?
-            // though it could be that some other code depends on this return code
-            // so I'm not changing anything, if you know better fix this! --mxcl
-            return 1;
+            return 0;
         }
     #endif
 
     #ifdef Q_WS_MAC
-        // on mac we do this because mac arguments with urls are handled wierdly
+        // on mac we do this because mac arguments with urls are handled weirdly
         // ask muesli/chris as he is our expert on this matter
         if ( !openUrl.isEmpty() )
         {
